@@ -982,6 +982,9 @@ template<class S> inline double area(const S& poly, const PolygonTag& )
     });
 }
 
+template<class RawShapes>
+inline double area(const RawShapes& shapes, const MultiPolygonTag&);
+
 template<class S> // Dispatching function
 inline double area(const S& sh)
 {
@@ -1144,7 +1147,7 @@ inline bool isInside(const TBGuest& ibb, const TBHost& box,
     auto minY = getY(box.minCorner());
     auto maxY = getY(box.maxCorner());
 
-    return iminX > minX && imaxX < maxX && iminY > minY && imaxY < maxY;
+    return iminX >= minX && imaxX <= maxX && iminY >= minY && imaxY <= maxY;
 }
 
 template<class S, class TB>
