@@ -741,6 +741,10 @@ public:
         return impl_.getResult();
     }
 
+    inline int lastPackedBinId() const {
+        return impl_.lastPackedBinId();
+    }
+
     void clear() { impl_.clear(); }
 };
 
@@ -862,6 +866,10 @@ public:
     {
         return selector_.getResult();
     }
+
+    inline int lastPackedBinId() const {
+        return selector_.lastPackedBinId();
+    }
 };
 
 }
