@@ -8,18 +8,26 @@ namespace Slic3r {
 
 class SpiralVase {
 public:
-    bool enable = false;
-    
     SpiralVase(const PrintConfig &config) : m_config(&config)
     {
         m_reader.z() = (float)m_config->z_offset;
         m_reader.apply_config(*m_config);
     };
+
+    void 		enable(bool en) {
+   		m_transition_layer = en && ! m_enabled;
+    	m_enabled 		   = en;
+    }
+
     std::string process_layer(const std::string &gcode);
     
 private:
     const PrintConfig  *m_config;
     GCodeReader 		m_reader;
+
+    bool 				m_enabled = false;
+    // First spiral vase layer. Layer height has to be ramped up from zero to the target layer height.
+    bool 				m_transition_layer = false;
 };
 
 }
