@@ -11,14 +11,15 @@
 
 #include "libslic3r.h"
 #include "Utils.hpp"
-#include "PrintConfig.hpp"
 
 namespace Slic3r
 {
 
 class PrintConfig;
 class PrintObjectConfig;
+class ModelConfig;
 class ModelObject;
+class DynamicPrintConfig;
 
 // Parameters to guide object slicing and support generation.
 // The slicing parameters account for a raft and whether the 1st object layer is printed with a normal or a bridging flow
@@ -128,9 +129,7 @@ inline bool equal_layering(const SlicingParameters &sp1, const SlicingParameters
 }
 
 typedef std::pair<coordf_t,coordf_t> t_layer_height_range;
-typedef std::map<t_layer_height_range, DynamicPrintConfig> t_layer_config_ranges;
-
-extern std::vector<std::pair<t_layer_height_range, coordf_t>> layer_height_ranges(const t_layer_config_ranges &config_ranges);
+typedef std::map<t_layer_height_range, ModelConfig> t_layer_config_ranges;
 
 extern std::vector<coordf_t> layer_height_profile_from_ranges(
     const SlicingParameters     &slicing_params,
