@@ -5,16 +5,18 @@
 #include "libslic3r/Polygon.hpp"
 #include "libslic3r/ClipperUtils.hpp"
 #include "libslic3r/BoundingBox.hpp"
+#include "libslic3r/Geometry.hpp"
 
 #include "GUI_App.hpp"
-#include "PresetBundle.hpp"
-#include "Gizmos/GLGizmoBase.hpp"
+#include "libslic3r/PresetBundle.hpp"
 #include "GLCanvas3D.hpp"
+#include "3DScene.hpp"
 
 #include <GL/glew.h>
 
 #include <boost/algorithm/string/predicate.hpp>
 #include <boost/filesystem/operations.hpp>
+#include <boost/log/trivial.hpp>
 
 static const float GROUND_Z = -0.02f;
 
@@ -37,10 +39,8 @@ bool GeometryBuffer::set_from_triangles(const Polygons& triangles, float z, bool
     float max_y = min_y;
 
     unsigned int v_count = 0;
-    for (const Polygon& t : triangles)
-    {
-        for (unsigned int i = 0; i < 3; ++i)
-        {
+    for (const Polygon& t : triangles) {
+        for (unsigned int i = 0; i < 3; ++i) {
             Vertex& v = m_vertices[v_count];
 
             const Point& p = t.points[i];
@@ -51,8 +51,7 @@ bool GeometryBuffer::set_from_triangles(const Polygons& triangles, float z, bool
             v.position[1] = y;
             v.position[2] = z;
 
-            if (generate_tex_coords)
-            {
+            if (generate_tex_coords) {
                 v.tex_coords[0] = x;
                 v.tex_coords[1] = y;
 
@@ -66,17 +65,14 @@ bool GeometryBuffer::set_from_triangles(const Polygons& triangles, float z, bool
         }
     }
 
-    if (generate_tex_coords)
-    {
+    if (generate_tex_coords) {
         float size_x = max_x - min_x;
         float size_y = max_y - min_y;
 
-        if ((size_x != 0.0f) && (size_y != 0.0f))
-        {
+        if ((size_x != 0.0f) && (size_y != 0.0f)) {
             float inv_size_x = 1.0f / size_x;
             float inv_size_y = -1.0f / size_y;
-            for (Vertex& v : m_vertices)
-            {
+            for (Vertex& v : m_vertices) {
                 v.tex_coords[0] = (v.tex_coords[0] - min_x) * inv_size_x;
                 v.tex_coords[1] = (v.tex_coords[1] - min_y) * inv_size_y;
             }
@@ -97,8 +93,7 @@ bool GeometryBuffer::set_from_lines(const Lines& lines, float z)
     m_vertices = std::vector<Vertex>(v_size, Vertex());
 
     unsigned int v_count = 0;
-    for (const Line& l : lines)
-    {
+    for (const Line& l : lines) {
         Vertex& v1 = m_vertices[v_count];
         v1.position[0] = unscale<float>(l.a(0));
         v1.position[1] = unscale<float>(l.a(1));
@@ -120,73 +115,56 @@ const float* GeometryBuffer::get_vertices_data() const
     return (m_vertices.size() > 0) ? (const float*)m_vertices.data() : nullptr;
 }
 
-const double Bed3D::Axes::Radius = 0.5;
-const double Bed3D::Axes::ArrowBaseRadius = 2.5 * Bed3D::Axes::Radius;
-const double Bed3D::Axes::ArrowLength = 5.0;
+const float Bed3D::Axes::DefaultStemRadius = 0.5f;
+const float Bed3D::Axes::DefaultStemLength = 25.0f;
+const float Bed3D::Axes::DefaultTipRadius = 2.5f * Bed3D::Axes::DefaultStemRadius;
+const float Bed3D::Axes::DefaultTipLength = 5.0f;
 
-Bed3D::Axes::Axes()
-: origin(Vec3d::Zero())
-, length(25.0 * Vec3d::Ones())
+void Bed3D::Axes::set_stem_length(float length)
 {
-    m_quadric = ::gluNewQuadric();
-    if (m_quadric != nullptr)
-        ::gluQuadricDrawStyle(m_quadric, GLU_FILL);
-}
-
-Bed3D::Axes::~Axes()
-{
-    if (m_quadric != nullptr)
-        ::gluDeleteQuadric(m_quadric);
+    m_stem_length = length;
+    m_arrow.reset();
 }
 
 void Bed3D::Axes::render() const
 {
-    if (m_quadric == nullptr)
+    auto render_axis = [this](const Transform3f& transform) {
+        glsafe(::glPushMatrix());
+        glsafe(::glMultMatrixf(transform.data()));
+        m_arrow.render();
+        glsafe(::glPopMatrix());
+    };
+
+    m_arrow.init_from(stilized_arrow(16, DefaultTipRadius, DefaultTipLength, DefaultStemRadius, m_stem_length));
+
+    GLShaderProgram* shader = wxGetApp().get_shader("gouraud_light");
+    if (shader == nullptr)
         return;
 
     glsafe(::glEnable(GL_DEPTH_TEST));
-    glsafe(::glEnable(GL_LIGHTING));
+
+    shader->start_using();
 
     // x axis
-    glsafe(::glColor3fv(AXES_COLOR[0]));
-    glsafe(::glPushMatrix());
-    glsafe(::glTranslated(origin(0), origin(1), origin(2)));
-    glsafe(::glRotated(90.0, 0.0, 1.0, 0.0));
-    render_axis(length(0));
-    glsafe(::glPopMatrix());
+    std::array<float, 4> color = { 0.75f, 0.0f, 0.0f, 1.0f };
+    shader->set_uniform("uniform_color", color);
+    render_axis(Geometry::assemble_transform(m_origin, { 0.0, 0.5 * M_PI, 0.0f }).cast<float>());
 
     // y axis
-    glsafe(::glColor3fv(AXES_COLOR[1]));
-    glsafe(::glPushMatrix());
-    glsafe(::glTranslated(origin(0), origin(1), origin(2)));
-    glsafe(::glRotated(-90.0, 1.0, 0.0, 0.0));
-    render_axis(length(1));
-    glsafe(::glPopMatrix());
+    color = { 0.0f, 0.75f, 0.0f, 1.0f };
+    shader->set_uniform("uniform_color", color);
+    render_axis(Geometry::assemble_transform(m_origin, { -0.5 * M_PI, 0.0, 0.0f }).cast<float>());
 
     // z axis
-    glsafe(::glColor3fv(AXES_COLOR[2]));
-    glsafe(::glPushMatrix());
-    glsafe(::glTranslated(origin(0), origin(1), origin(2)));
-    render_axis(length(2));
-    glsafe(::glPopMatrix());
+    color = { 0.0f, 0.0f, 0.75f, 1.0f };
+    shader->set_uniform("uniform_color", color);
+    render_axis(Geometry::assemble_transform(m_origin).cast<float>());
+
+    shader->stop_using();
 
-    glsafe(::glDisable(GL_LIGHTING));
     glsafe(::glDisable(GL_DEPTH_TEST));
 }
 
-void Bed3D::Axes::render_axis(double length) const
-{
-    ::gluQuadricOrientation(m_quadric, GLU_OUTSIDE);
-    ::gluCylinder(m_quadric, Radius, Radius, length, 32, 1);
-    ::gluQuadricOrientation(m_quadric, GLU_INSIDE);
-    ::gluDisk(m_quadric, 0.0, Radius, 32, 1);
-    glsafe(::glTranslated(0.0, 0.0, length));
-    ::gluQuadricOrientation(m_quadric, GLU_OUTSIDE);
-    ::gluCylinder(m_quadric, ArrowBaseRadius, 0.0, ArrowLength, 32, 1);
-    ::gluQuadricOrientation(m_quadric, GLU_INSIDE);
-    ::gluDisk(m_quadric, 0.0, ArrowBaseRadius, 32, 1);
-}
-
 Bed3D::Bed3D()
     : m_type(Custom)
     , m_vbo_id(0)
@@ -194,7 +172,7 @@ Bed3D::Bed3D()
 {
 }
 
-bool Bed3D::set_shape(const Pointfs& shape, const std::string& custom_texture, const std::string& custom_model)
+bool Bed3D::set_shape(const Pointfs& shape, const std::string& custom_texture, const std::string& custom_model, bool force_as_custom)
 {
     auto check_texture = [](const std::string& texture) {
         return !texture.empty() && (boost::algorithm::iends_with(texture, ".png") || boost::algorithm::iends_with(texture, ".svg")) && boost::filesystem::exists(texture);
@@ -204,30 +182,39 @@ bool Bed3D::set_shape(const Pointfs& shape, const std::string& custom_texture, c
         return !model.empty() && boost::algorithm::iends_with(model, ".stl") && boost::filesystem::exists(model);
     };
 
-    auto [new_type, system_model, system_texture] = detect_type(shape);
+    EType type;
+    std::string model;
+    std::string texture;
+    if (force_as_custom)
+        type = Custom;
+    else {
+        auto [new_type, system_model, system_texture] = detect_type(shape);
+        type = new_type;
+        model = system_model;
+        texture = system_texture;
+    }
 
-    std::string texture_filename = custom_texture.empty() ? system_texture : custom_texture;
+    std::string texture_filename = custom_texture.empty() ? texture : custom_texture;
     if (!check_texture(texture_filename))
         texture_filename.clear();
 
-    std::string model_filename = custom_model.empty() ? system_model : custom_model;
+    std::string model_filename = custom_model.empty() ? model : custom_model;
     if (!check_model(model_filename))
         model_filename.clear();
 
-    if ((m_shape == shape) && (m_type == new_type) && (m_texture_filename == texture_filename) && (m_model_filename == model_filename))
+    if (m_shape == shape && m_type == type && m_texture_filename == texture_filename && m_model_filename == model_filename)
         // No change, no need to update the UI.
         return false;
 
     m_shape = shape;
     m_texture_filename = texture_filename;
     m_model_filename = model_filename;
-    m_type = new_type;
+    m_type = type;
 
     calc_bounding_boxes();
 
     ExPolygon poly;
-    for (const Vec2d& p : m_shape)
-    {
+    for (const Vec2d& p : m_shape) {
         poly.contour.append(Point(scale_(p(0)), scale_(p(1))));
     }
 
@@ -243,8 +230,8 @@ bool Bed3D::set_shape(const Pointfs& shape, const std::string& custom_texture, c
     m_model.reset();
 
     // Set the origin and size for rendering the coordinate system axes.
-    m_axes.origin = Vec3d(0.0, 0.0, (double)GROUND_Z);
-    m_axes.length = 0.1 * m_bounding_box.max_size() * Vec3d::Ones();
+    m_axes.set_origin({ 0.0, 0.0, static_cast<double>(GROUND_Z) });
+    m_axes.set_stem_length(0.1f * static_cast<float>(m_bounding_box.max_size()));
 
     // Let the calee to update the UI.
     return true;
@@ -260,7 +247,8 @@ Point Bed3D::point_projection(const Point& point) const
     return m_polygon.point_projection(point);
 }
 
-void Bed3D::render(GLCanvas3D& canvas, bool bottom, float scale_factor, bool show_axes) const
+void Bed3D::render(GLCanvas3D& canvas, bool bottom, float scale_factor,
+                   bool show_axes, bool show_texture) const
 {
     m_scale_factor = scale_factor;
 
@@ -271,9 +259,9 @@ void Bed3D::render(GLCanvas3D& canvas, bool bottom, float scale_factor, bool sho
 
     switch (m_type)
     {
-    case System: { render_system(canvas, bottom); break; }
+    case System: { render_system(canvas, bottom, show_texture); break; }
     default:
-    case Custom: { render_custom(canvas, bottom); break; }
+    case Custom: { render_custom(canvas, bottom, show_texture); break; }
     }
 
     glsafe(::glDisable(GL_DEPTH_TEST));
@@ -282,25 +270,28 @@ void Bed3D::render(GLCanvas3D& canvas, bool bottom, float scale_factor, bool sho
 void Bed3D::calc_bounding_boxes() const
 {
     m_bounding_box = BoundingBoxf3();
-    for (const Vec2d& p : m_shape)
-    {
+    for (const Vec2d& p : m_shape) {
         m_bounding_box.merge(Vec3d(p(0), p(1), 0.0));
     }
 
     m_extended_bounding_box = m_bounding_box;
 
     // extend to contain axes
-    m_extended_bounding_box.merge(m_axes.length + Axes::ArrowLength * Vec3d::Ones());
+    m_extended_bounding_box.merge(m_axes.get_origin() + m_axes.get_total_length() * Vec3d::Ones());
+    m_extended_bounding_box.merge(m_extended_bounding_box.min + Vec3d(-Axes::DefaultTipRadius, -Axes::DefaultTipRadius, m_extended_bounding_box.max(2)));
 
     // extend to contain model, if any
-    if (!m_model.get_filename().empty())
-        m_extended_bounding_box.merge(m_model.get_transformed_bounding_box());
+    BoundingBoxf3 model_bb = m_model.get_bounding_box();
+    if (model_bb.defined) {
+        model_bb.translate(m_model_offset);
+        m_extended_bounding_box.merge(model_bb);
+    }
 }
 
 void Bed3D::calc_triangles(const ExPolygon& poly)
 {
     Polygons triangles;
-    poly.triangulate(&triangles);
+    poly.triangulate_p2t(&triangles);
 
     if (!m_triangles.set_from_triangles(triangles, GROUND_Z, true))
         printf("Unable to create bed triangles\n");
@@ -309,15 +300,13 @@ void Bed3D::calc_triangles(const ExPolygon& poly)
 void Bed3D::calc_gridlines(const ExPolygon& poly, const BoundingBox& bed_bbox)
 {
     Polylines axes_lines;
-    for (coord_t x = bed_bbox.min(0); x <= bed_bbox.max(0); x += scale_(10.0))
-    {
+    for (coord_t x = bed_bbox.min(0); x <= bed_bbox.max(0); x += scale_(10.0)) {
         Polyline line;
         line.append(Point(x, bed_bbox.min(1)));
         line.append(Point(x, bed_bbox.max(1)));
         axes_lines.push_back(line);
     }
-    for (coord_t y = bed_bbox.min(1); y <= bed_bbox.max(1); y += scale_(10.0))
-    {
+    for (coord_t y = bed_bbox.min(1); y <= bed_bbox.max(1); y += scale_(10.0)) {
         Polyline line;
         line.append(Point(bed_bbox.min(0), y));
         line.append(Point(bed_bbox.max(0), y));
@@ -335,40 +324,19 @@ void Bed3D::calc_gridlines(const ExPolygon& poly, const BoundingBox& bed_bbox)
         printf("Unable to create bed grid lines\n");
 }
 
-static std::string system_print_bed_model(const Preset &preset)
-{
-	std::string out;
-	const VendorProfile::PrinterModel *pm = PresetUtils::system_printer_model(preset);
-	if (pm != nullptr && ! pm->bed_model.empty())
-		out = Slic3r::resources_dir() + "/profiles/" + preset.vendor->id + "/" + pm->bed_model;
-	return out;
-}
-
-static std::string system_print_bed_texture(const Preset &preset)
-{
-	std::string out;
-	const VendorProfile::PrinterModel *pm = PresetUtils::system_printer_model(preset);
-	if (pm != nullptr && ! pm->bed_texture.empty())
-		out = Slic3r::resources_dir() + "/profiles/" + preset.vendor->id + "/" + pm->bed_texture;
-	return out;
-}
 
 std::tuple<Bed3D::EType, std::string, std::string> Bed3D::detect_type(const Pointfs& shape) const
 {
     auto bundle = wxGetApp().preset_bundle;
-    if (bundle != nullptr)
-    {
+    if (bundle != nullptr) {
         const Preset* curr = &bundle->printers.get_selected_preset();
-        while (curr != nullptr)
-        {
-            if (curr->config.has("bed_shape"))
-            {
-                if (shape == dynamic_cast<const ConfigOptionPoints*>(curr->config.option("bed_shape"))->values)
-                {
-                    std::string model_filename = system_print_bed_model(*curr);
-                    std::string texture_filename = system_print_bed_texture(*curr);
+        while (curr != nullptr) {
+            if (curr->config.has("bed_shape")) {
+                if (shape == dynamic_cast<const ConfigOptionPoints*>(curr->config.option("bed_shape"))->values) {
+                    std::string model_filename = PresetUtils::system_printer_bed_model(*curr);
+                    std::string texture_filename = PresetUtils::system_printer_bed_texture(*curr);
                     if (!model_filename.empty() && !texture_filename.empty())
-                        return std::make_tuple(System, model_filename, texture_filename);
+                        return { System, model_filename, texture_filename };
                 }
             }
 
@@ -376,7 +344,7 @@ std::tuple<Bed3D::EType, std::string, std::string> Bed3D::detect_type(const Poin
         }
     }
 
-    return std::make_tuple(Custom, "", "");
+    return { Custom, "", "" };
 }
 
 void Bed3D::render_axes() const
@@ -385,36 +353,32 @@ void Bed3D::render_axes() const
         m_axes.render();
 }
 
-void Bed3D::render_system(GLCanvas3D& canvas, bool bottom) const
+void Bed3D::render_system(GLCanvas3D& canvas, bool bottom, bool show_texture) const
 {
     if (!bottom)
         render_model();
 
-    render_texture(bottom, canvas);
+    if (show_texture)
+        render_texture(bottom, canvas);
 }
 
 void Bed3D::render_texture(bool bottom, GLCanvas3D& canvas) const
 {
-    if (m_texture_filename.empty())
-    {
+    if (m_texture_filename.empty()) {
         m_texture.reset();
         render_default(bottom);
         return;
     }
 
-    if ((m_texture.get_id() == 0) || (m_texture.get_source() != m_texture_filename))
-    {
+    if ((m_texture.get_id() == 0) || (m_texture.get_source() != m_texture_filename)) {
         m_texture.reset();
 
-        if (boost::algorithm::iends_with(m_texture_filename, ".svg"))
-        {
+        if (boost::algorithm::iends_with(m_texture_filename, ".svg")) {
             // use higher resolution images if graphic card and opengl version allow
-            GLint max_tex_size = GLCanvas3DManager::get_gl_info().get_max_tex_size();
-            if ((m_temp_texture.get_id() == 0) || (m_temp_texture.get_source() != m_texture_filename))
-            {
+            GLint max_tex_size = OpenGLManager::get_gl_info().get_max_tex_size();
+            if ((m_temp_texture.get_id() == 0) || (m_temp_texture.get_source() != m_texture_filename)) {
                 // generate a temporary lower resolution texture to show while no main texture levels have been compressed
-                if (!m_temp_texture.load_from_svg_file(m_texture_filename, false, false, false, max_tex_size / 8))
-                {
+                if (!m_temp_texture.load_from_svg_file(m_texture_filename, false, false, false, max_tex_size / 8)) {
                     render_default(bottom);
                     return;
                 }
@@ -422,19 +386,15 @@ void Bed3D::render_texture(bool bottom, GLCanvas3D& canvas) const
             }
 
             // starts generating the main texture, compression will run asynchronously
-            if (!m_texture.load_from_svg_file(m_texture_filename, true, true, true, max_tex_size))
-            {
+            if (!m_texture.load_from_svg_file(m_texture_filename, true, true, true, max_tex_size)) {
                 render_default(bottom);
                 return;
             }
-        }
-        else if (boost::algorithm::iends_with(m_texture_filename, ".png"))
-        {
+        } 
+        else if (boost::algorithm::iends_with(m_texture_filename, ".png")) {
             // generate a temporary lower resolution texture to show while no main texture levels have been compressed
-            if ((m_temp_texture.get_id() == 0) || (m_temp_texture.get_source() != m_texture_filename))
-            {
-                if (!m_temp_texture.load_from_file(m_texture_filename, false, GLTexture::None, false))
-                {
+            if ((m_temp_texture.get_id() == 0) || (m_temp_texture.get_source() != m_texture_filename)) {
+                if (!m_temp_texture.load_from_file(m_texture_filename, false, GLTexture::None, false)) {
                     render_default(bottom);
                     return;
                 }
@@ -442,20 +402,17 @@ void Bed3D::render_texture(bool bottom, GLCanvas3D& canvas) const
             }
 
             // starts generating the main texture, compression will run asynchronously
-            if (!m_texture.load_from_file(m_texture_filename, true, GLTexture::MultiThreaded, true))
-            {
+            if (!m_texture.load_from_file(m_texture_filename, true, GLTexture::MultiThreaded, true)) {
                 render_default(bottom);
                 return;
             }
         }
-        else
-        {
+        else {
             render_default(bottom);
             return;
         }
     }
-    else if (m_texture.unsent_compressed_data_available())
-    {
+    else if (m_texture.unsent_compressed_data_available()) {
         // sends to gpu the already available compressed levels of the main texture
         m_texture.send_compressed_data_to_gpu();
 
@@ -467,19 +424,14 @@ void Bed3D::render_texture(bool bottom, GLCanvas3D& canvas) const
 
     }
 
-    if (m_triangles.get_vertices_count() > 0)
-    {
-        if (m_shader.get_shader_program_id() == 0)
-            m_shader.init("printbed.vs", "printbed.fs");
+    if (m_triangles.get_vertices_count() > 0) {
+        GLShaderProgram* shader = wxGetApp().get_shader("printbed");
+        if (shader != nullptr) {
+            shader->start_using();
+            shader->set_uniform("transparent_background", bottom);
+            shader->set_uniform("svg_source", boost::algorithm::iends_with(m_texture.get_source(), ".svg"));
 
-        if (m_shader.is_initialized())
-        {
-            m_shader.start_using();
-            m_shader.set_uniform("transparent_background", bottom);
-            m_shader.set_uniform("svg_source", boost::algorithm::iends_with(m_texture.get_source(), ".svg"));
-
-            if (m_vbo_id == 0)
-            {
+            if (m_vbo_id == 0) {
                 glsafe(::glGenBuffers(1, &m_vbo_id));
                 glsafe(::glBindBuffer(GL_ARRAY_BUFFER, m_vbo_id));
                 glsafe(::glBufferData(GL_ARRAY_BUFFER, (GLsizeiptr)m_triangles.get_vertices_data_size(), (const GLvoid*)m_triangles.get_vertices_data(), GL_STATIC_DRAW));
@@ -497,8 +449,8 @@ void Bed3D::render_texture(bool bottom, GLCanvas3D& canvas) const
 
             unsigned int stride = m_triangles.get_vertex_data_size();
 
-            GLint position_id = m_shader.get_attrib_location("v_position");
-            GLint tex_coords_id = m_shader.get_attrib_location("v_tex_coords");
+            GLint position_id = shader->get_attrib_location("v_position");
+            GLint tex_coords_id = shader->get_attrib_location("v_tex_coords");
 
             // show the temporary texture while no compressed data is available
             GLuint tex_id = (GLuint)m_temp_texture.get_id();
@@ -508,13 +460,11 @@ void Bed3D::render_texture(bool bottom, GLCanvas3D& canvas) const
             glsafe(::glBindTexture(GL_TEXTURE_2D, tex_id));
             glsafe(::glBindBuffer(GL_ARRAY_BUFFER, m_vbo_id));
 
-            if (position_id != -1)
-            {
+            if (position_id != -1) {
                 glsafe(::glEnableVertexAttribArray(position_id));
                 glsafe(::glVertexAttribPointer(position_id, 3, GL_FLOAT, GL_FALSE, stride, (GLvoid*)(intptr_t)m_triangles.get_position_offset()));
             }
-            if (tex_coords_id != -1)
-            {
+            if (tex_coords_id != -1) {
                 glsafe(::glEnableVertexAttribArray(tex_coords_id));
                 glsafe(::glVertexAttribPointer(tex_coords_id, 2, GL_FLOAT, GL_FALSE, stride, (GLvoid*)(intptr_t)m_triangles.get_tex_coords_offset()));
             }
@@ -536,7 +486,7 @@ void Bed3D::render_texture(bool bottom, GLCanvas3D& canvas) const
             glsafe(::glDisable(GL_BLEND));
             glsafe(::glDepthMask(GL_TRUE));
 
-            m_shader.stop_using();
+            shader->stop_using();
         }
     }
 }
@@ -546,29 +496,33 @@ void Bed3D::render_model() const
     if (m_model_filename.empty())
         return;
 
-    if ((m_model.get_filename() != m_model_filename) && m_model.init_from_file(m_model_filename))
-    {
+    if ((m_model.get_filename() != m_model_filename) && m_model.init_from_file(m_model_filename)) {
         // move the model so that its origin (0.0, 0.0, 0.0) goes into the bed shape center and a bit down to avoid z-fighting with the texture quad
         Vec3d shift = m_bounding_box.center();
         shift(2) = -0.03;
-        m_model.set_offset(shift);
+        m_model_offset = shift;
 
         // update extended bounding box
         calc_bounding_boxes();
     }
 
-    if (!m_model.get_filename().empty())
-    {
-        glsafe(::glEnable(GL_LIGHTING));
-        m_model.render();
-        glsafe(::glDisable(GL_LIGHTING));
+    if (!m_model.get_filename().empty()) {
+        GLShaderProgram* shader = wxGetApp().get_shader("gouraud_light");
+        if (shader != nullptr) {
+            shader->start_using();
+            shader->set_uniform("uniform_color", m_model_color);
+            ::glPushMatrix();
+            ::glTranslated(m_model_offset(0), m_model_offset(1), m_model_offset(2));
+            m_model.render();
+            ::glPopMatrix();
+            shader->stop_using();
+        }
     }
 }
 
-void Bed3D::render_custom(GLCanvas3D& canvas, bool bottom) const
+void Bed3D::render_custom(GLCanvas3D& canvas, bool bottom, bool show_texture) const
 {
-    if (m_texture_filename.empty() && m_model_filename.empty())
-    {
+    if (m_texture_filename.empty() && m_model_filename.empty()) {
         render_default(bottom);
         return;
     }
@@ -576,7 +530,8 @@ void Bed3D::render_custom(GLCanvas3D& canvas, bool bottom) const
     if (!bottom)
         render_model();
 
-    render_texture(bottom, canvas);
+    if (show_texture)
+        render_texture(bottom, canvas);
 }
 
 void Bed3D::render_default(bool bottom) const
@@ -584,8 +539,7 @@ void Bed3D::render_default(bool bottom) const
     m_texture.reset();
 
     unsigned int triangles_vcount = m_triangles.get_vertices_count();
-    if (triangles_vcount > 0)
-    {
+    if (triangles_vcount > 0) {
         bool has_model = !m_model.get_filename().empty();
 
         glsafe(::glEnable(GL_DEPTH_TEST));
@@ -594,11 +548,10 @@ void Bed3D::render_default(bool bottom) const
 
         glsafe(::glEnableClientState(GL_VERTEX_ARRAY));
 
-        if (!has_model && !bottom)
-        {
+        if (!has_model && !bottom) {
             // draw background
             glsafe(::glDepthMask(GL_FALSE));
-            glsafe(::glColor4f(0.35f, 0.35f, 0.35f, 0.4f));
+            glsafe(::glColor4fv(m_model_color.data()));
             glsafe(::glNormal3d(0.0f, 0.0f, 1.0f));
             glsafe(::glVertexPointer(3, GL_FLOAT, m_triangles.get_vertex_data_size(), (GLvoid*)m_triangles.get_vertices_data()));
             glsafe(::glDrawArrays(GL_TRIANGLES, 0, (GLsizei)triangles_vcount));
@@ -606,11 +559,11 @@ void Bed3D::render_default(bool bottom) const
         }
 
         // draw grid
-        glsafe(::glLineWidth(3.0f * m_scale_factor));
+        glsafe(::glLineWidth(1.5f * m_scale_factor));
         if (has_model && !bottom)
-            glsafe(::glColor4f(0.75f, 0.75f, 0.75f, 1.0f));
+            glsafe(::glColor4f(0.9f, 0.9f, 0.9f, 1.0f));
         else
-            glsafe(::glColor4f(0.2f, 0.2f, 0.2f, 0.4f));
+            glsafe(::glColor4f(0.9f, 0.9f, 0.9f, 0.6f));
         glsafe(::glVertexPointer(3, GL_FLOAT, m_triangles.get_vertex_data_size(), (GLvoid*)m_gridlines.get_vertices_data()));
         glsafe(::glDrawArrays(GL_LINES, 0, (GLsizei)m_gridlines.get_vertices_count()));
 
@@ -622,12 +575,11 @@ void Bed3D::render_default(bool bottom) const
 
 void Bed3D::reset()
 {
-    if (m_vbo_id > 0)
-    {
+    if (m_vbo_id > 0) {
         glsafe(::glDeleteBuffers(1, &m_vbo_id));
         m_vbo_id = 0;
     }
 }
 
 } // GUI
-} // Slic3r
\ No newline at end of file
+} // Slic3r
