@@ -6,6 +6,7 @@
 #include "libslic3r/Utils.hpp"
 #include "libslic3r/Time.hpp"
 #include "GUI_App.hpp"
+#include "MainFrame.hpp"
 #include "wxExtensions.hpp"
 
 namespace Slic3r { 
@@ -48,9 +49,17 @@ static wxString generate_html_row(const Config::Snapshot &snapshot, bool row_eve
     text += "</b></font><br>";
     // End of row header.
     text += _(L("PrusaSlicer version")) + ": " + snapshot.slic3r_version_captured.to_string() + "<br>";
-    text += _(L("print")) + ": " + snapshot.print + "<br>";
-    text += _(L("filaments")) + ": " + snapshot.filaments.front() + "<br>";
-    text += _(L("printer")) + ": " + snapshot.printer + "<br>";
+    bool has_fff = ! snapshot.print.empty() || ! snapshot.filaments.empty();
+    bool has_sla = ! snapshot.sla_print.empty() || ! snapshot.sla_material.empty();
+    if (has_fff || ! has_sla) {
+        text += _(L("print")) + ": " + snapshot.print + "<br>";
+        text += _(L("filaments")) + ": " + snapshot.filaments.front() + "<br>";
+    }
+    if (has_sla) {
+        text += _(L("SLA print")) + ": " + snapshot.sla_print + "<br>";
+        text += _(L("SLA material")) + ": " + snapshot.sla_material + "<br>";
+    }
+    text += _(L("printer")) + ": " + (snapshot.physical_printer.empty() ? snapshot.printer : snapshot.physical_printer) + "<br>";
 
     bool compatible = true;
     for (const Config::Snapshot::VendorConfig &vc : snapshot.vendor_configs) {
@@ -101,7 +110,7 @@ static wxString generate_html_page(const Config::SnapshotDB &snapshot_db, const
 }
 
 ConfigSnapshotDialog::ConfigSnapshotDialog(const Config::SnapshotDB &snapshot_db, const wxString &on_snapshot)
-    : DPIDialog(NULL, wxID_ANY, _(L("Configuration Snapshots")), wxDefaultPosition, 
+    : DPIDialog(static_cast<wxWindow*>(wxGetApp().mainframe), wxID_ANY, _(L("Configuration Snapshots")), wxDefaultPosition,
                wxSize(45 * wxGetApp().em_unit(), 40 * wxGetApp().em_unit()), 
                wxDEFAULT_DIALOG_STYLE | wxRESIZE_BORDER | wxMAXIMIZE_BOX)
 {
@@ -114,7 +123,7 @@ ConfigSnapshotDialog::ConfigSnapshotDialog(const Config::SnapshotDB &snapshot_db
     // text
     html = new wxHtmlWindow(this, wxID_ANY, wxDefaultPosition, wxDefaultSize, wxHW_SCROLLBAR_AUTO);
     {
-        wxFont font = wxGetApp().normal_font();//wxSystemSettings::GetFont(wxSYS_DEFAULT_GUI_FONT);
+        wxFont font = get_default_font(this);
         #ifdef __WXMSW__
             const int fs = font.GetPointSize();
             const int fs1 = static_cast<int>(0.8f*fs);
@@ -140,7 +149,7 @@ ConfigSnapshotDialog::ConfigSnapshotDialog(const Config::SnapshotDB &snapshot_db
 
 void ConfigSnapshotDialog::on_dpi_changed(const wxRect &suggested_rect)
 {
-    wxFont font = GetFont();
+    wxFont font = get_default_font(this);
     const int fs = font.GetPointSize();
     const int fs1 = static_cast<int>(0.8f*fs);
     const int fs2 = static_cast<int>(1.1f*fs);
