@@ -13,6 +13,8 @@
 #include "wxExtensions.hpp"
 #include "ObjectDataViewModel.hpp"
 
+#include "libslic3r/PrintConfig.hpp"
+
 class wxBoxSizer;
 class wxBitmapComboBox;
 class wxMenuItem;
@@ -22,8 +24,10 @@ class MenuWithSeparators;
 namespace Slic3r {
 class ConfigOptionsGroup;
 class DynamicPrintConfig;
+class ModelConfig;
 class ModelObject;
 class ModelVolume;
+class TriangleMesh;
 enum class ModelVolumeType : int;
 
 // FIXME: broken build on mac os because of this is missing:
@@ -36,9 +40,9 @@ typedef std::map< std::string, std::vector< std::pair<std::string, std::string>
 
 typedef std::vector<ModelVolume*> ModelVolumePtrs;
 
-typedef double                                              coordf_t;
-typedef std::pair<coordf_t, coordf_t>                       t_layer_height_range;
-typedef std::map<t_layer_height_range, DynamicPrintConfig>  t_layer_config_ranges;
+typedef double                                       coordf_t;
+typedef std::pair<coordf_t, coordf_t>                t_layer_height_range;
+typedef std::map<t_layer_height_range, ModelConfig>  t_layer_config_ranges;
 
 namespace GUI {
 
@@ -80,10 +84,32 @@ public:
         smLayerRoot = 16, // used for undo/redo
     };
 
+    struct Clipboard
+    {
+        void reset() {
+            m_type = itUndef; 
+            m_layer_config_ranges_cache .clear();
+            m_config_cache.clear();
+        }
+        bool        empty()    const { return m_type == itUndef; }
+        ItemType    get_type() const { return m_type; }
+        void        set_type(ItemType type) { m_type = type; }
+
+        t_layer_config_ranges&  get_ranges_cache() { return m_layer_config_ranges_cache; }
+        DynamicPrintConfig&     get_config_cache() { return m_config_cache; }
+
+    private:
+        ItemType                m_type {itUndef};
+        t_layer_config_ranges   m_layer_config_ranges_cache;
+        DynamicPrintConfig      m_config_cache;
+    };
+
 private:
     SELECTION_MODE  m_selection_mode {smUndef};
     int             m_selected_layers_range_idx;
 
+    Clipboard       m_clipboard;
+
     struct dragged_item_data
     {
         void init(const int obj_idx, const int subobj_idx, const ItemType type) {
@@ -140,15 +166,13 @@ private:
     wxMenuItem* m_menu_item_split_instances { nullptr };
 
     ObjectDataViewModel         *m_objects_model{ nullptr };
-    DynamicPrintConfig          *m_config {nullptr};
+    ModelConfig                 *m_config {nullptr};
     std::vector<ModelObject*>   *m_objects{ nullptr };
 
     wxBitmapComboBox            *m_extruder_editor { nullptr };
 
     std::vector<wxBitmap*>      m_bmp_vector;
 
-    t_layer_config_ranges       m_layer_config_ranges_cache;
-
     int			m_selected_object_id = -1;
     bool		m_prevent_list_events = false;		// We use this flag to avoid circular event handling Select() 
                                                     // happens to fire a wxEVT_LIST_ITEM_SELECTED on OSX, whose event handler 
@@ -183,11 +207,12 @@ public:
     ObjectList(wxWindow* parent);
     ~ObjectList();
 
+    void set_min_height();
 
     std::map<std::string, wxBitmap> CATEGORY_ICON;
 
     ObjectDataViewModel*        GetModel() const    { return m_objects_model; }
-    DynamicPrintConfig*         config() const      { return m_config; }
+    ModelConfig*                config() const      { return m_config; }
     std::vector<ModelObject*>*  objects() const     { return m_objects; }
 
     ModelObject*                object(const int obj_idx) const ;
@@ -209,6 +234,7 @@ public:
 
     // Get obj_idx and vol_idx values for the selected (by default) or an adjusted item
     void                get_selected_item_indexes(int& obj_idx, int& vol_idx, const wxDataViewItem& item = wxDataViewItem(0));
+    void                get_selection_indexes(std::vector<int>& obj_idxs, std::vector<int>& vol_idxs);
     // Get count of errors in the mesh
     int                 get_mesh_errors_count(const int obj_idx, const int vol_idx = -1) const;
     /* Get list of errors in the mesh. Return value is a string, used for the tooltip
@@ -228,8 +254,12 @@ public:
 
     void                copy();
     void                paste();
+    bool                copy_to_clipboard();
+    bool                paste_from_clipboard();
     void                undo();
     void                redo();
+    void                increase_instances();
+    void                decrease_instances();
 
     void                get_settings_choice(const wxString& category_name);
     void                get_freq_settings_choice(const wxString& bundle_name);
@@ -251,6 +281,9 @@ public:
     void                append_menu_item_change_extruder(wxMenu* menu);
     void                append_menu_item_delete(wxMenu* menu);
     void                append_menu_item_scale_selection_to_fit_print_volume(wxMenu* menu);
+    void                append_menu_item_convert_unit(wxMenu* menu, int insert_pos = 1); // Add "Conver/Revert..." menu item after "Reload From Disk"
+    void                append_menu_item_merge_to_multipart_object(wxMenu *menu);
+    void                append_menu_item_merge_to_single_object(wxMenu *menu);
     void                create_object_popupmenu(wxMenu *menu);
     void                create_sla_object_popupmenu(wxMenu*menu);
     void                create_part_popupmenu(wxMenu*menu);
@@ -265,6 +298,7 @@ public:
     void                load_part(ModelObject* model_object, std::vector<std::pair<wxString, bool>> &volumes_info, ModelVolumeType type);
 	void                load_generic_subobject(const std::string& type_name, const ModelVolumeType type);
     void                load_shape_object(const std::string &type_name);
+    void                load_mesh_object(const TriangleMesh &mesh, const wxString &name, bool center = true);
     void                del_object(const int obj_idx);
     void                del_subobject_item(wxDataViewItem& item);
     void                del_settings_from_config(const wxDataViewItem& parent_item);
@@ -273,6 +307,7 @@ public:
     void                del_layers_from_object(const int obj_idx);
     bool                del_subobject_from_object(const int obj_idx, const int idx, const int type);
     void                split();
+    void                merge(bool to_multipart_object);
     void                layers_editing();
 
     wxDataViewItem      add_layer_root_item(const wxDataViewItem obj_item);
@@ -283,11 +318,13 @@ public:
     bool                is_splittable();
     bool                selected_instances_of_same_object();
     bool                can_split_instances();
+    bool                can_merge_to_multipart_object() const;
+    bool                can_merge_to_single_object() const;
 
     wxPoint             get_mouse_position_in_control() const { return wxGetMousePosition() - this->GetScreenPosition(); }
     wxBoxSizer*         get_sizer() {return  m_sizer;}
     int                 get_selected_obj_idx() const;
-    DynamicPrintConfig& get_item_config(const wxDataViewItem& item) const;
+    ModelConfig&        get_item_config(const wxDataViewItem& item) const;
     SettingsBundle      get_item_settings_bundle(const DynamicPrintConfig* config, const bool is_object_settings);
 
     void                changed_object(const int obj_idx = -1) const;
@@ -351,6 +388,7 @@ public:
     void update_selections();
     void update_selections_on_canvas();
     void select_item(const wxDataViewItem& item);
+    void select_item(std::function<wxDataViewItem()> get_item);
     void select_items(const wxDataViewItemArray& sels);
     void select_all();
     void select_item_all_children();
@@ -376,12 +414,16 @@ public:
     void fix_through_netfabb();
     void update_item_error_icon(const int obj_idx, int vol_idx) const ;
 
-    void fill_layer_config_ranges_cache();
+    void copy_layers_to_clipboard();
     void paste_layers_into_list();
+    void copy_settings_to_clipboard();
+    void paste_settings_into_list();
+    bool clipboard_is_empty() const { return m_clipboard.empty(); } 
     void paste_volumes_into_list(int obj_idx, const ModelVolumePtrs& volumes);
     void paste_objects_into_list(const std::vector<size_t>& object_idxs);
 
     void msw_rescale();
+    void sys_color_changed();
 
     void update_after_undo_redo();
     //update printable state for item from objects model
