@@ -10,6 +10,7 @@ class wxBoxSizer;
 
 namespace Slic3r {
 class DynamicPrintConfig;
+class ModelConfig;
 namespace GUI {
 class ConfigOptionsGroup;
 
@@ -52,8 +53,8 @@ public:
      * Example: if Infill is set to 100%, and Fill Pattern is missed in config_to,
      * we should add fill_pattern to avoid endless loop in update
      */
-    bool        add_missed_options(DynamicPrintConfig *config_to, const DynamicPrintConfig &config_from);
-    void        update_config_values(DynamicPrintConfig*config);
+    bool        add_missed_options(ModelConfig *config_to, const DynamicPrintConfig &config_from);
+    void        update_config_values(ModelConfig *config);
     void        UpdateAndShow(const bool show) override;
     void        msw_rescale();
 };
