@@ -8,6 +8,10 @@
 
 #include "libslic3r/Point.hpp"
 
+namespace Slic3r {namespace Search {
+struct OptionViewParameters;
+}}
+
 class wxString;
 class wxMouseEvent;
 class wxKeyEvent;
@@ -53,6 +57,7 @@ public:
 
     void set_next_window_pos(float x, float y, int flag, float pivot_x = 0.0f, float pivot_y = 0.0f);
     void set_next_window_bg_alpha(float alpha);
+	void set_next_window_size(float x, float y, ImGuiCond cond);
 
     bool begin(const std::string &name, int flags = 0);
     bool begin(const wxString &name, int flags = 0);
@@ -61,7 +66,9 @@ public:
     void end();
 
     bool button(const wxString &label);
+	bool button(const wxString& label, float width, float height);
     bool radio_button(const wxString &label, bool active);
+	bool image_button();
     bool input_double(const std::string &label, const double &value, const std::string &format = "%.3f");
     bool input_double(const wxString &label, const double &value, const std::string &format = "%.3f");
     bool input_vec3(const std::string &label, const Vec3d &value, float width, const std::string &format = "%.3f");
@@ -69,11 +76,17 @@ public:
     void text(const char *label);
     void text(const std::string &label);
     void text(const wxString &label);
+    void text_colored(const ImVec4& color, const char* label);
+    void text_colored(const ImVec4& color, const std::string& label);
+    void text_colored(const ImVec4& color, const wxString& label);
     bool slider_float(const char* label, float* v, float v_min, float v_max, const char* format = "%.3f", float power = 1.0f);
     bool slider_float(const std::string& label, float* v, float v_min, float v_max, const char* format = "%.3f", float power = 1.0f);
     bool slider_float(const wxString& label, float* v, float v_min, float v_max, const char* format = "%.3f", float power = 1.0f);
     bool combo(const wxString& label, const std::vector<std::string>& options, int& selection);   // Use -1 to not mark any option as selected
-    bool undo_redo_list(const ImVec2& size, const bool is_undo, bool (*items_getter)(const bool, int, const char**), int& hovered, int& selected);
+    bool undo_redo_list(const ImVec2& size, const bool is_undo, bool (*items_getter)(const bool, int, const char**), int& hovered, int& selected, int& mouse_wheel);
+    void search_list(const ImVec2& size, bool (*items_getter)(int, const char** label, const char** tooltip), char* search_str,
+                     Search::OptionViewParameters& view_params, int& selected, bool& edited, int& mouse_wheel, bool is_localized);
+    void title(const std::string& str);
 
     void disabled_begin(bool disabled);
     void disabled_end();
@@ -83,6 +96,15 @@ public:
     bool want_text_input() const;
     bool want_any_input() const;
 
+    static const ImVec4 COL_GREY_DARK;
+    static const ImVec4 COL_GREY_LIGHT;
+    static const ImVec4 COL_ORANGE_DARK;
+    static const ImVec4 COL_ORANGE_LIGHT;
+    static const ImVec4 COL_WINDOW_BACKGROUND;
+    static const ImVec4 COL_BUTTON_BACKGROUND;
+    static const ImVec4 COL_BUTTON_HOVERED;
+    static const ImVec4 COL_BUTTON_ACTIVE;
+
 private:
     void init_font(bool compress);
     void init_input();
@@ -90,6 +112,7 @@ private:
     void render_draw_data(ImDrawData *draw_data);
     bool display_initialized() const;
     void destroy_font();
+    std::vector<unsigned char> load_svg(const std::string& bitmap_name, unsigned target_width, unsigned target_height);
 
     static const char* clipboard_get(void* user_data);
     static void clipboard_set(void* user_data, const char* text);
