@@ -64,12 +64,9 @@ MsgDialog::MsgDialog(wxWindow *parent, const wxString &title, const wxString &he
 	SetSizerAndFit(topsizer);
 }
 
-MsgDialog::~MsgDialog() {}
-
-
 // ErrorDialog
 
-ErrorDialog::ErrorDialog(wxWindow *parent, const wxString &msg)
+ErrorDialog::ErrorDialog(wxWindow *parent, const wxString &msg, bool monospaced_font)
     : MsgDialog(parent, wxString::Format(_(L("%s error")), SLIC3R_APP_NAME), 
                         wxString::Format(_(L("%s has encountered an error")), SLIC3R_APP_NAME),
 		wxID_NONE)
@@ -78,19 +75,23 @@ ErrorDialog::ErrorDialog(wxWindow *parent, const wxString &msg)
     // Text shown as HTML, so that mouse selection and Ctrl-V to copy will work.
     wxHtmlWindow* html = new wxHtmlWindow(this, wxID_ANY, wxDefaultPosition, wxDefaultSize, wxHW_SCROLLBAR_AUTO);
     {
-        html->SetMinSize(wxSize(40 * wxGetApp().em_unit(), -1));
+        html->SetMinSize(wxSize(40 * wxGetApp().em_unit(), monospaced_font ? 30 * wxGetApp().em_unit() : -1));
         wxFont 	  	font 			= wxSystemSettings::GetFont(wxSYS_DEFAULT_GUI_FONT);
+        wxFont      monospace       = wxGetApp().code_font();
 		wxColour  	text_clr  		= wxSystemSettings::GetColour(wxSYS_COLOUR_WINDOWTEXT);
         wxColour  	bgr_clr 		= wxSystemSettings::GetColour(wxSYS_COLOUR_WINDOW);
 		auto      	text_clr_str 	= wxString::Format(wxT("#%02X%02X%02X"), text_clr.Red(), text_clr.Green(), text_clr.Blue());
 		auto      	bgr_clr_str 	= wxString::Format(wxT("#%02X%02X%02X"), bgr_clr.Red(), bgr_clr.Green(), bgr_clr.Blue());
 		const int 	font_size       = font.GetPointSize()-1;
         int 		size[] 			= {font_size, font_size, font_size, font_size, font_size, font_size, font_size};
-        html->SetFonts(font.GetFaceName(), font.GetFaceName(), size);
+        html->SetFonts(font.GetFaceName(), monospace.GetFaceName(), size);
         html->SetBorders(2);
 		std::string msg_escaped = xml_escape(msg.ToUTF8().data());
 		boost::replace_all(msg_escaped, "\r\n", "<br>");
         boost::replace_all(msg_escaped, "\n", "<br>");
+		if (monospaced_font)
+			// Code formatting will be preserved. This is useful for reporting errors from the placeholder parser.
+			msg_escaped = std::string("<pre><code>") + msg_escaped + "</code></pre>";
 		html->SetPage("<html><body bgcolor=\"" + bgr_clr_str + "\"><font color=\"" + text_clr_str + "\">" + wxString::FromUTF8(msg_escaped.data()) + "</font></body></html>");
 		content_sizer->Add(html, 1, wxEXPAND);
     }
@@ -99,15 +100,12 @@ ErrorDialog::ErrorDialog(wxWindow *parent, const wxString &msg)
 	btn_ok->SetFocus();
 	btn_sizer->Add(btn_ok, 0, wxRIGHT, HORIZ_SPACING);
 
-	logo->SetBitmap(create_scaled_bitmap("PrusaSlicer_192px_grayscale.png", this, 192));
+	// Use a small bitmap with monospaced font, as the error text will not be wrapped.
+	logo->SetBitmap(create_scaled_bitmap("PrusaSlicer_192px_grayscale.png", this, monospaced_font ? 48 : 192));
 
     SetMaxSize(wxSize(-1, CONTENT_MAX_HEIGHT*wxGetApp().em_unit()));
 	Fit();
 }
 
-ErrorDialog::~ErrorDialog() {}
-
-
-
 }
 }
