@@ -11,7 +11,7 @@
 #include "libslic3r/PrintConfig.hpp"
 
 #include "Field.hpp"
-#include "GUI_App.hpp"
+#include "I18N.hpp"
 
 // Translate the ifdef 
 #ifdef __WXOSX__
@@ -24,12 +24,13 @@
 
 namespace Slic3r { namespace GUI {
 
+// Thrown if the building of a parameter page is canceled.
+class UIBuildCanceled : public std::exception {};
+class OG_CustomCtrl;
+
 /// Widget type describes a function object that returns a wxWindow (our widget) and accepts a wxWidget (parent window).
 using widget_t = std::function<wxSizer*(wxWindow*)>;//!std::function<wxWindow*(wxWindow*)>;
 
-//auto default_label_clr = wxSystemSettings::GetColour(wxSYS_COLOUR_3DLIGHT); //GetSystemColour
-//auto modified_label_clr = *new wxColour(254, 189, 101);
-
 /// Wraps a ConfigOptionDef and adds function object for creating a side_widget.
 struct Option {
 	ConfigOptionDef			opt { ConfigOptionDef() };
@@ -37,6 +38,10 @@ struct Option {
     widget_t				side_widget {nullptr};
     bool					readonly {false};
 
+	bool operator==(const Option& rhs) const {
+		return  (rhs.opt_id == this->opt_id);
+	}
+
 	Option(const ConfigOptionDef& _opt, t_config_option_key id) :
 		opt(_opt), opt_id(id) {}
 };
@@ -45,12 +50,18 @@ using t_option = std::unique_ptr<Option>;	//!
 /// Represents option lines
 class Line {
 public:
-    wxString	label {wxString("")};
-    wxString	label_tooltip {wxString("")};
+    wxString	label;
+    wxString	label_tooltip;
+	wxString	label_path;
+
     size_t		full_width {0}; 
-    wxSizer*	sizer {nullptr};
+	wxColour*	full_Label_color {nullptr};
+	bool		blink	{false};
     widget_t	widget {nullptr};
     std::function<wxWindow*(wxWindow*)>	near_label_widget{ nullptr };
+	wxWindow*	near_label_widget_win {nullptr};
+    wxSizer*	widget_sizer {nullptr};
+    wxSizer*	extra_widget_sizer {nullptr};
 
     void append_option(const Option& option) {
         m_options.push_back(option);
@@ -59,10 +70,11 @@ public:
 		m_extra_widgets.push_back(widget);
     }
 	Line(wxString label, wxString tooltip) :
-		label(label), label_tooltip(tooltip) {}
+		label(_(label)), label_tooltip(_(tooltip)) {}
 
     const std::vector<widget_t>&	get_extra_widgets() const {return m_extra_widgets;}
     const std::vector<Option>&		get_options() const { return m_options; }
+	bool*							get_blink_ptr() { return &blink; }
 
 private:
 	std::vector<Option>		m_options;//! {std::vector<Option>()};
@@ -78,9 +90,10 @@ class OptionsGroup {
 	wxStaticBox*	stb;
 public:
     const bool		staticbox {true};
-    const wxString	title {wxString("")};
+    const wxString	title;
     size_t			label_width = 20 ;// {200};
     wxSizer*		sizer {nullptr};
+	OG_CustomCtrl*  custom_ctrl{ nullptr };
     column_t		extra_column {nullptr};
     t_change		m_on_change { nullptr };
 	// To be called when the field loses focus, to assign a new initial value to the field.
@@ -102,26 +115,21 @@ public:
     /// Returns a copy of the pointer of the parent wxWindow.
     /// Accessor function is because users are not allowed to change the parent
     /// but defining it as const means a lot of const_casts to deal with wx functions.
-    inline wxWindow* parent() const { 
-#if 0//#ifdef __WXGTK__
-		return m_panel;
-#else
-		return m_parent;
-#endif /* __WXGTK__ */
-    }
-#if 0//#ifdef __WXGTK__
-    wxWindow* get_parent() const {
-        return m_parent;
-    }
-#endif /* __WXGTK__ */
+    inline wxWindow* parent() const { return m_parent; }
 
-    wxWindow* ctrl_parent() const {
-    	return this->stb ? (wxWindow*)this->stb : this->parent();
-    }
+    wxWindow*   ctrl_parent() const;
 
-	void		append_line(const Line& line, wxStaticText** full_Label = nullptr);
-    Line		create_single_option_line(const Option& option) const;
-    void		append_single_option_line(const Option& option) { append_line(create_single_option_line(option)); }
+	void		append_line(const Line& line);
+	// create controls for the option group
+	void		activate_line(Line& line);
+
+	// create all controls for the option group from the m_lines
+	bool		activate(std::function<void()> throw_if_canceled = [](){});
+	// delete all controls from the option group
+	void		clear(bool destroy_custom_ctrl = false);
+
+    Line		create_single_option_line(const Option& option, const wxString& path = wxEmptyString) const;
+    void		append_single_option_line(const Option& option, const wxString& path = wxEmptyString) { append_line(create_single_option_line(option, path)); }
 
     // return a non-owning pointer reference 
     inline Field*	get_field(const t_config_option_key& id) const{
@@ -141,13 +149,8 @@ public:
 							return out;
     }
 
-	bool			set_side_text(const t_config_option_key& opt_key, const wxString& side_text) {
-							if (m_fields.find(opt_key) == m_fields.end()) return false;
-							auto st = m_fields.at(opt_key)->m_side_text;
-							if (!st) return false;
-							st->SetLabel(side_text);
-							return true;
-    }
+	void			show_field(const t_config_option_key& opt_key, bool show = true);
+	void			hide_field(const t_config_option_key& opt_key) {  show_field(opt_key, false);  }
 
 	void			set_name(const wxString& new_name) {
 							stb->SetLabel(new_name);
@@ -157,53 +160,25 @@ public:
     inline void		disable() { for (auto& field : m_fields) field.second->disable(); }
 	void			set_grid_vgap(int gap) { m_grid_sizer->SetVGap(gap); }
 
-	void            set_show_modified_btns_val(bool show) {
-		                m_show_modified_btns = show;
-    }
-
     void            clear_fields_except_of(const std::vector<std::string> left_fields);
 
-    void            hide_labels() {
-        label_width = 0;
-        m_grid_sizer->SetCols(m_grid_sizer->GetEffectiveColsCount()-1);
-        static_cast<wxFlexGridSizer*>(m_grid_sizer)->AddGrowableCol(!extra_column ? 0 : 1);
-    }
+    void            hide_labels() { label_width = 0; }
 
 	OptionsGroup(	wxWindow* _parent, const wxString& title, bool is_tab_opt = false, 
-					column_t extra_clmn = nullptr) :
-					m_parent(_parent), title(title), 
-                    m_show_modified_btns(is_tab_opt),
-					staticbox(title!=""), extra_column(extra_clmn) {
-        if (staticbox) {
-            stb = new wxStaticBox(_parent, wxID_ANY, title);
-            if (!wxOSX) stb->SetBackgroundStyle(wxBG_STYLE_PAINT);
-            stb->SetFont(wxGetApp().bold_font());
-        } else
-        	stb = nullptr;
-        sizer = (staticbox ? new wxStaticBoxSizer(stb, wxVERTICAL) : new wxBoxSizer(wxVERTICAL));
-        auto num_columns = 1U;
-        if (label_width != 0) num_columns++;
-        if (extra_column != nullptr) num_columns++;
-        m_grid_sizer = new wxFlexGridSizer(0, num_columns, 1,0);
-        static_cast<wxFlexGridSizer*>(m_grid_sizer)->SetFlexibleDirection(wxBOTH/*wxHORIZONTAL*/);
-        static_cast<wxFlexGridSizer*>(m_grid_sizer)->AddGrowableCol(label_width == 0 ? 0 : !extra_column ? 1 : 2 );
-#if 0//#ifdef __WXGTK__
-        m_panel = new wxPanel( _parent, wxID_ANY, wxDefaultPosition, wxDefaultSize, wxTAB_TRAVERSAL );
-        sizer->Fit(m_panel);
-        sizer->Add(m_panel, 0, wxEXPAND | wxALL, wxOSX||!staticbox ? 0: 5);
-#else
-        sizer->Add(m_grid_sizer, 0, wxEXPAND | wxALL, wxOSX||!staticbox ? 0: 5);
-#endif /* __WXGTK__ */
-    }
+                    column_t extra_clmn = nullptr);
+	~OptionsGroup() { clear(true); }
 
     wxGridSizer*        get_grid_sizer() { return m_grid_sizer; }
+	const std::vector<Line>& get_lines() { return m_lines; }
+	bool				is_legend_line();
 
 protected:
 	std::map<t_config_option_key, Option>	m_options;
     wxWindow*				m_parent {nullptr};
     std::vector<ConfigOptionMode>           m_options_mode;
     std::vector<wxWindow*>                  m_extra_column_item_ptrs;
-    std::vector<wxWindow*>                  m_near_label_widget_ptrs;
+
+    std::vector<Line>                       m_lines;
 
     /// Field list, contains unique_ptrs of the derived type.
     /// using types that need to know what it is beyond the public interface 
@@ -212,7 +187,10 @@ protected:
     bool					m_disabled {false};
     wxGridSizer*			m_grid_sizer {nullptr};
 	// "true" if option is created in preset tabs
-	bool					m_show_modified_btns{ false };
+	bool					m_use_custom_ctrl{ false };
+
+	// "true" if control should be created on custom_ctrl
+	bool					m_use_custom_ctrl_as_parent { false };
 
 	// This panel is needed for correct showing of the ToolTips for Button, StaticText and CheckBox
 	// Tooltips on GTK doesn't work inside wxStaticBoxSizer unless you insert a panel 
@@ -224,10 +202,9 @@ protected:
     /// Generate a wxSizer or wxWindow from a configuration option
     /// Precondition: opt resolves to a known ConfigOption
     /// Postcondition: fields contains a wx gui object.
-	const t_field&		build_field(const t_config_option_key& id, const ConfigOptionDef& opt, wxStaticText* label = nullptr);
-	const t_field&		build_field(const t_config_option_key& id, wxStaticText* label = nullptr);
-	const t_field&		build_field(const Option& opt, wxStaticText* label = nullptr);
-	void				add_undo_buttuns_to_sizer(wxSizer* sizer, const t_field& field);
+	const t_field&		build_field(const t_config_option_key& id, const ConfigOptionDef& opt);
+	const t_field&		build_field(const t_config_option_key& id);
+	const t_field&		build_field(const Option& opt);
 
     virtual void		on_kill_focus(const std::string& opt_key) {};
 	virtual void		on_set_focus(const std::string& opt_key);
@@ -238,31 +215,35 @@ protected:
 
 class ConfigOptionsGroup: public OptionsGroup {
 public:
-	ConfigOptionsGroup(	wxWindow* parent, const wxString& title, DynamicPrintConfig* _config = nullptr, 
+	ConfigOptionsGroup(	wxWindow* parent, const wxString& title, DynamicPrintConfig* config = nullptr, 
 						bool is_tab_opt = false, column_t extra_clmn = nullptr) :
-		OptionsGroup(parent, title, is_tab_opt, extra_clmn), m_config(_config) {}
+		OptionsGroup(parent, title, is_tab_opt, extra_clmn), m_config(config) {}
+	ConfigOptionsGroup(	wxWindow* parent, const wxString& title, ModelConfig* config, 
+						bool is_tab_opt = false, column_t extra_clmn = nullptr) :
+		OptionsGroup(parent, title, is_tab_opt, extra_clmn), m_config(&config->get()), m_modelconfig(config) {}
+	ConfigOptionsGroup(	wxWindow* parent) :
+		OptionsGroup(parent, wxEmptyString, true, nullptr) {}
 
-    /// reference to libslic3r config, non-owning pointer (?).
-    DynamicPrintConfig*		m_config {nullptr};
-    bool					m_full_labels {0};
-	t_opt_map				m_opt_map;
+	const std::string& config_category() const throw() { return m_config_category; }
+	const t_opt_map&   opt_map() const throw() { return m_opt_map; }
 
-    void        set_config(DynamicPrintConfig* config) { m_config = config; }
+	void 		set_config_category(const std::string &category) { this->m_config_category = category; }
+    void        set_config(DynamicPrintConfig* config) { m_config = config; m_modelconfig = nullptr; }
 	Option		get_option(const std::string& opt_key, int opt_index = -1);
-	Line		create_single_option_line(const std::string& title, int idx = -1) /*const*/{
+	Line		create_single_option_line(const std::string& title, const wxString& path = wxEmptyString, int idx = -1) /*const*/{
 		Option option = get_option(title, idx);
-		return OptionsGroup::create_single_option_line(option);
+		return OptionsGroup::create_single_option_line(option, path);
 	}
-	Line		create_single_option_line(const Option& option) const {
-		return OptionsGroup::create_single_option_line(option);
+	Line		create_single_option_line(const Option& option, const wxString& path = wxEmptyString) const {
+		return OptionsGroup::create_single_option_line(option, path);
 	}
-	void		append_single_option_line(const Option& option)	{
-		OptionsGroup::append_single_option_line(option);
+	void		append_single_option_line(const Option& option, const wxString& path = wxEmptyString)	{
+		OptionsGroup::append_single_option_line(option, path);
 	}
-	void		append_single_option_line(const std::string title, int idx = -1)
+	void		append_single_option_line(const std::string title, const wxString& path = wxEmptyString, int idx = -1)
 	{
 		Option option = get_option(title, idx);
-		append_single_option_line(option);		
+		append_single_option_line(option, path);
 	}
 
 	void		on_change_OG(const t_config_option_key& opt_id, const boost::any& value) override;
@@ -274,19 +255,37 @@ public:
     // return value shows visibility : false => all options are hidden
     void        Hide();
     void        Show(const bool show);
+    bool        is_visible(ConfigOptionMode mode);
     bool        update_visibility(ConfigOptionMode mode);
     void        msw_rescale();
+    void        sys_color_changed();
+    void        refresh();
 	boost::any	config_value(const std::string& opt_key, int opt_index, bool deserialize);
 	// return option value from config 
 	boost::any	get_config_value(const DynamicPrintConfig& config, const std::string& opt_key, int opt_index = -1);
 	Field*		get_fieldc(const t_config_option_key& opt_key, int opt_index);
+	std::pair<OG_CustomCtrl*, bool*>	get_custom_ctrl_with_blinking_ptr(const t_config_option_key& opt_key, int opt_index/* = -1*/);
+
+private:
+    // Reference to libslic3r config or ModelConfig::get(), non-owning pointer.
+    // The reference is const, so that the spots which modify m_config are clearly
+    // demarcated by const_cast and m_config_changed_callback is called afterwards.
+    const DynamicPrintConfig*	m_config {nullptr};
+    // If the config is modelconfig, then ModelConfig::touch() has to be called after value change.
+    ModelConfig*				m_modelconfig { nullptr };
+	bool						m_full_labels{ 0 };
+	t_opt_map					m_opt_map;
+    std::string             	m_config_category;
+
+    // Change an option on m_config, possibly call ModelConfig::touch().
+	void 	change_opt_value(const t_config_option_key& opt_key, const boost::any& value, int opt_index = 0);
 };
 
 //  Static text shown among the options.
 class ogStaticText :public wxStaticText{
 public:
 	ogStaticText() {}
-	ogStaticText(wxWindow* parent, const char *text) : wxStaticText(parent, wxID_ANY, text, wxDefaultPosition, wxDefaultSize) {}
+	ogStaticText(wxWindow* parent, const wxString& text);
 	~ogStaticText() {}
 
 	void		SetText(const wxString& value, bool wrap = true);
