@@ -6,18 +6,17 @@
 #include <boost/filesystem/path.hpp>
 
 #include <wx/panel.h>
-#include <wx/bmpcbox.h>
 
-#include "Preset.hpp"
+#include "Selection.hpp"
 
-#include "3DScene.hpp"
-#include "GLTexture.hpp"
-#include "wxExtensions.hpp"
+#include "libslic3r/Preset.hpp"
+#include "libslic3r/BoundingBox.hpp"
+#include "libslic3r/GCode/GCodeProcessor.hpp"
+#include "Jobs/Job.hpp"
+#include "Search.hpp"
 
 class wxButton;
 class ScalableButton;
-class wxBoxSizer;
-class wxGLCanvas;
 class wxScrolledWindow;
 class wxString;
 
@@ -25,14 +24,17 @@ namespace Slic3r {
 
 class Model;
 class ModelObject;
+class ModelInstance;
 class Print;
 class SLAPrint;
 enum SLAPrintObjectStep : unsigned int;
 
+using ModelInstancePtrs = std::vector<ModelInstance*>;
+
 namespace UndoRedo {
-	class Stack;
-	struct Snapshot;	
-};
+    class Stack;
+    struct Snapshot;
+}
 
 namespace GUI {
 
@@ -44,46 +46,17 @@ class ObjectLayers;
 class ObjectList;
 class GLCanvas3D;
 class Mouse3DController;
+class NotificationManager;
+struct Camera;
+class Bed3D;
+class GLToolbar;
+class PlaterPresetComboBox;
 
 using t_optgroups = std::vector <std::shared_ptr<ConfigOptionsGroup>>;
 
 class Plater;
 enum class ActionButtonType : int;
 
-class PresetComboBox : public PresetBitmapComboBox
-{
-public:
-    PresetComboBox(wxWindow *parent, Preset::Type preset_type);
-    ~PresetComboBox();
-
-    ScalableButton* edit_btn { nullptr };
-
-	enum LabelItemType {
-		LABEL_ITEM_MARKER = 0xffffff01,
-		LABEL_ITEM_WIZARD_PRINTERS,
-        LABEL_ITEM_WIZARD_FILAMENTS,
-        LABEL_ITEM_WIZARD_MATERIALS,
-
-        LABEL_ITEM_MAX,
-	};
-
-    void set_label_marker(int item, LabelItemType label_item_type = LABEL_ITEM_MARKER);
-    void set_extruder_idx(const int extr_idx)   { extruder_idx = extr_idx; }
-    int  get_extruder_idx() const               { return extruder_idx; }
-    int  em_unit() const                        { return m_em_unit; }
-    void check_selection();
-
-    void msw_rescale();
-
-private:
-    typedef std::size_t Marker;
-
-    Preset::Type preset_type;
-    int last_selected;
-    int extruder_idx = -1;
-    int m_em_unit;
-};
-
 class Sidebar : public wxPanel
 {
     ConfigOptionMode    m_mode;
@@ -95,13 +68,17 @@ public:
     Sidebar &operator=(const Sidebar &) = delete;
     ~Sidebar();
 
-    void init_filament_combo(PresetComboBox **combo, const int extr_idx);
+    void init_filament_combo(PlaterPresetComboBox **combo, const int extr_idx);
     void remove_unused_filament_combos(const size_t current_extruder_count);
     void update_all_preset_comboboxes();
     void update_presets(Slic3r::Preset::Type preset_type);
     void update_mode_sizer() const;
+    void change_top_border_for_mode_sizer(bool increase_border);
     void update_reslice_btn_tooltip() const;
     void msw_rescale();
+    void sys_color_changed();
+    void search();
+    void jump_to_option(size_t selected);
 
     ObjectManipulation*     obj_manipul();
     ObjectList*             obj_list();
@@ -121,12 +98,20 @@ public:
     bool                    show_reslice(bool show) const;
 	bool                    show_export(bool show) const;
 	bool                    show_send(bool show) const;
-    bool                    show_disconnect(bool show)const;
+    bool                    show_eject(bool show)const;
 	bool                    show_export_removable(bool show) const;
+	bool                    get_eject_shown() const;
     bool                    is_multifilament();
     void                    update_mode();
+    bool                    is_collapsed();
+    void                    collapse(bool collapse);
+    void                    update_searcher();
+    void                    update_ui_from_settings();
+
+    std::vector<PlaterPresetComboBox*>&   combos_filament();
+    Search::OptionsSearcher&        get_searcher();
+    std::string&                    get_search_line();
 
-    std::vector<PresetComboBox*>& combos_filament();
 private:
     struct priv;
     std::unique_ptr<priv> p;
@@ -154,12 +139,23 @@ public:
     void new_project();
     void load_project();
     void load_project(const wxString& filename);
-    void add_model();
+    void add_model(bool imperial_units = false);
+    void import_sl1_archive();
     void extract_config_from_project();
+    void load_gcode();
+    void load_gcode(const wxString& filename);
+    void reload_gcode_from_disk();
+    void refresh_print();
 
-    std::vector<size_t> load_files(const std::vector<boost::filesystem::path>& input_files, bool load_model = true, bool load_config = true);
+    std::vector<size_t> load_files(const std::vector<boost::filesystem::path>& input_files, bool load_model = true, bool load_config = true, bool imperial_units = false);
     // To be called when providing a list of files to the GUI slic3r on command line.
-    std::vector<size_t> load_files(const std::vector<std::string>& input_files, bool load_model = true, bool load_config = true);
+    std::vector<size_t> load_files(const std::vector<std::string>& input_files, bool load_model = true, bool load_config = true, bool imperial_units = false);
+#if ENABLE_DRAG_AND_DROP_FIX
+    // to be called on drag and drop
+    bool load_files(const wxArrayString& filenames);
+#endif // ENABLE_DRAG_AND_DROP_FIX
+
+    const wxString& get_last_loaded_gcode() const { return m_last_loaded_gcode; }
 
     void update();
     void stop_jobs();
@@ -173,9 +169,14 @@ public:
     bool are_view3D_labels_shown() const;
     void show_view3D_labels(bool show);
 
+    bool is_sidebar_collapsed() const;
+    void collapse_sidebar(bool show);
+
+    bool is_view3D_layers_editing_enabled() const;
+
     // Called after the Preferences dialog is closed and the program settings are saved.
     // Update the UI based on the current preferences.
-    void update_ui_from_settings();
+    void update_ui_from_settings(bool apply_free_camera_correction = true);
 
     void select_all();
     void deselect_all();
@@ -187,12 +188,14 @@ public:
     void increase_instances(size_t num = 1);
     void decrease_instances(size_t num = 1);
     void set_number_of_copies(/*size_t num*/);
+    void fill_bed_with_instances();
     bool is_selection_empty() const;
     void scale_selection_to_fit_print_volume();
+    void convert_unit(bool from_imperial_unit);
 
     void cut(size_t obj_idx, size_t instance_idx, coordf_t z, bool keep_upper = true, bool keep_lower = true, bool rotate_lower = false);
 
-    void export_gcode(bool prefer_removable = true);
+    void export_gcode(bool prefer_removable);
     void export_stl(bool extended = false, bool selection_only = false);
     void export_amf();
     void export_3mf(const boost::filesystem::path& output_path = boost::filesystem::path());
@@ -200,7 +203,6 @@ public:
     void reload_all_from_disk();
     bool has_toolpaths_to_export() const;
     void export_toolpaths_to_obj() const;
-    void hollow();
     void reslice();
     void reslice_SLA_supports(const ModelObject &object, bool postpone_error_messages = false);
     void reslice_SLA_hollowing(const ModelObject &object, bool postpone_error_messages = false);
@@ -222,8 +224,10 @@ public:
     void redo_to(int selection);
     bool undo_redo_string_getter(const bool is_undo, int idx, const char** out_text);
     void undo_redo_topmost_string_getter(const bool is_undo, std::string& out_text);
+    bool search_string_getter(int idx, const char** label, const char** tooltip);
     // For the memory statistics. 
     const Slic3r::UndoRedo::Stack& undo_redo_stack_main() const;
+    void clear_undo_redo_stack_main();
     // Enter / leave the Gizmos specific Undo / Redo stack. To be used by the SLA support point editing gizmo.
     void enter_gizmos_stack();
     void leave_gizmos_stack();
@@ -234,8 +238,8 @@ public:
     void force_print_bed_update();
     // On activating the parent window.
     void on_activate();
-    std::vector<std::string> get_extruder_colors_from_plater_config() const;
-    std::vector<std::string> get_colors_for_color_print() const;
+    std::vector<std::string> get_extruder_colors_from_plater_config(const GCodeProcessor::Result* const result = nullptr) const;
+    std::vector<std::string> get_colors_for_color_print(const GCodeProcessor::Result* const result = nullptr) const;
 
     void update_object_menu();
     void show_action_buttons(const bool is_ready_to_slice) const;
@@ -244,20 +248,29 @@ public:
     void set_project_filename(const wxString& filename);
 
     bool is_export_gcode_scheduled() const;
-
+    
+    const Selection& get_selection() const;
     int get_selected_object_idx();
     bool is_single_full_object_selection() const;
     GLCanvas3D* canvas3D();
+    const GLCanvas3D * canvas3D() const;
     GLCanvas3D* get_current_canvas3D();
     BoundingBoxf bed_shape_bb() const;
+    
+    void arrange();
+    void find_new_position(const ModelInstancePtrs  &instances, coord_t min_d);
 
     void set_current_canvas_as_dirty();
+    void unbind_canvas_event_handlers();
+    void reset_canvas_volumes();
 
     PrinterTechnology   printer_technology() const;
-    void                set_printer_technology(PrinterTechnology printer_technology);
+    const DynamicPrintConfig * config() const;
+    bool                set_printer_technology(PrinterTechnology printer_technology);
 
     void copy_selection_to_clipboard();
     void paste_from_clipboard();
+    void search(bool plater_is_active);
 
     bool can_delete() const;
     bool can_delete_all() const;
@@ -276,17 +289,49 @@ public:
     bool can_reload_from_disk() const;
 
     void msw_rescale();
+    void sys_color_changed();
 
     bool init_view_toolbar();
+    void enable_view_toolbar(bool enable);
+    bool init_collapse_toolbar();
+    void enable_collapse_toolbar(bool enable);
 
     const Camera& get_camera() const;
     Camera& get_camera();
+
+#if ENABLE_ENVIRONMENT_MAP
+    void init_environment_texture();
+    unsigned int get_environment_texture_id() const;
+#endif // ENABLE_ENVIRONMENT_MAP
+
+    const Bed3D& get_bed() const;
+    Bed3D& get_bed();
+
+    const GLToolbar& get_view_toolbar() const;
+    GLToolbar& get_view_toolbar();
+
+    const GLToolbar& get_collapse_toolbar() const;
+    GLToolbar& get_collapse_toolbar();
+
+    void update_preview_bottom_toolbar();
+    void update_preview_moves_slider();
+    void enable_preview_moves_slider(bool enable);
+
+    void reset_gcode_toolpaths();
+    void reset_last_loaded_gcode() { m_last_loaded_gcode = ""; }
+
     const Mouse3DController& get_mouse3d_controller() const;
     Mouse3DController& get_mouse3d_controller();
 
 	void set_bed_shape() const;
+    void set_bed_shape(const Pointfs& shape, const std::string& custom_texture, const std::string& custom_model, bool force_as_custom = false) const;
 
-	// ROII wrapper for suppressing the Undo / Redo snapshot to be taken.
+	const NotificationManager* get_notification_manager() const;
+	NotificationManager* get_notification_manager();
+
+    void bring_instance_forward();
+
+    // ROII wrapper for suppressing the Undo / Redo snapshot to be taken.
 	class SuppressSnapshots
 	{
 	public:
@@ -321,6 +366,11 @@ public:
 
     bool inside_snapshot_capture();
 
+#if ENABLE_RENDER_STATISTICS
+    void toggle_render_statistic_dialog();
+    bool is_render_statistic_dialog_visible() const;
+#endif // ENABLE_RENDER_STATISTICS
+
 	// Wrapper around wxWindow::PopupMenu to suppress error messages popping out while tracking the popup menu.
 	bool PopupMenu(wxMenu *menu, const wxPoint& pos = wxDefaultPosition);
     bool PopupMenu(wxMenu *menu, int x, int y) { return this->PopupMenu(menu, wxPoint(x, y)); }
@@ -335,6 +385,8 @@ private:
     bool 	 m_tracking_popup_menu = false;
     wxString m_tracking_popup_menu_error_message;
 
+    wxString m_last_loaded_gcode;
+
     void suppress_snapshots();
     void allow_snapshots();
 
@@ -350,6 +402,7 @@ private:
     bool m_was_scheduled;
 };
 
-}}
+} // namespace GUI
+} // namespace Slic3r
 
 #endif
