@@ -7,6 +7,8 @@
 #include <wx/dialog.h>
 #include <map>
 
+class wxRadioBox;
+
 namespace Slic3r {
 namespace GUI {
 
@@ -18,12 +20,21 @@ class PreferencesDialog : public DPIDialog
 	std::shared_ptr<ConfigOptionsGroup>	m_optgroup_general;
 	std::shared_ptr<ConfigOptionsGroup>	m_optgroup_camera;
 	std::shared_ptr<ConfigOptionsGroup>	m_optgroup_gui;
+#if ENABLE_ENVIRONMENT_MAP
+	std::shared_ptr<ConfigOptionsGroup>	m_optgroup_render;
+#endif // ENABLE_ENVIRONMENT_MAP
 	wxSizer*                            m_icon_size_sizer;
+	wxRadioBox*							m_layout_mode_box;
     bool                                isOSX {false};
+	bool								m_settings_layout_changed {false};
+	bool								m_seq_top_layer_only_changed{ false };
 public:
 	PreferencesDialog(wxWindow* parent);
 	~PreferencesDialog() {}
 
+	bool settings_layout_changed() const { return m_settings_layout_changed; }
+	bool seq_top_layer_only_changed() const { return m_seq_top_layer_only_changed; }
+
 	void	build();
 	void	accept();
 
@@ -31,6 +42,7 @@ protected:
     void on_dpi_changed(const wxRect &suggested_rect) override;
     void layout();
     void create_icon_size_slider();
+    void create_settings_mode_widget();
 };
 
 } // GUI
