@@ -30,10 +30,10 @@
 
 #include "libslic3r/Model.hpp"
 #include "libslic3r/Print.hpp"
+#include "libslic3r/PresetBundle.hpp"
 #include "libslic3r/Format/3mf.hpp"
 #include "../GUI/GUI.hpp"
 #include "../GUI/I18N.hpp"
-#include "../GUI/PresetBundle.hpp"
 
 #include <wx/msgdlg.h>
 #include <wx/progdlg.h>
@@ -209,10 +209,10 @@ typedef std::function<void (const char * /* message */, unsigned /* progress */)
 void fix_model_by_win10_sdk(const std::string &path_src, const std::string &path_dst, ProgressFn on_progress, ThrowOnCancelFn throw_on_cancel)
 {
 	if (! is_windows10())
-		throw std::runtime_error("fix_model_by_win10_sdk called on non Windows 10 system");
+		throw Slic3r::RuntimeError("fix_model_by_win10_sdk called on non Windows 10 system");
 
 	if (! winrt_load_runtime_object_library())
-		throw std::runtime_error("Failed to initialize the WinRT library.");
+		throw Slic3r::RuntimeError("Failed to initialize the WinRT library.");
 
 	HRESULT hr = (*s_RoInitialize)(RO_INIT_MULTITHREADED);
 	{
@@ -232,7 +232,7 @@ void fix_model_by_win10_sdk(const std::string &path_src, const std::string &path
 		if (status == AsyncStatus::Completed)
 			hr = modelAsync->GetResults(model.GetAddressOf());
 		else
-			throw std::runtime_error(L("Failed loading the input model."));
+			throw Slic3r::RuntimeError(L("Failed loading the input model."));
 
 		Microsoft::WRL::ComPtr<ABI::Windows::Foundation::Collections::IVector<ABI::Windows::Graphics::Printing3D::Printing3DMesh*>> meshes;
 		hr = model->get_Meshes(meshes.GetAddressOf());
@@ -245,7 +245,7 @@ void fix_model_by_win10_sdk(const std::string &path_src, const std::string &path
 		hr = model->RepairAsync(repairAsync.GetAddressOf());
 		status = winrt_async_await(repairAsync, throw_on_cancel);
 		if (status != AsyncStatus::Completed)
-			throw std::runtime_error(L("Mesh repair failed."));
+			throw Slic3r::RuntimeError(L("Mesh repair failed."));
 		repairAsync->GetResults();
 
 		on_progress(L("Loading repaired model"), 60);
@@ -260,14 +260,14 @@ void fix_model_by_win10_sdk(const std::string &path_src, const std::string &path
 		hr = printing3d3mfpackage->SaveModelToPackageAsync(model.Get(), saveToPackageAsync.GetAddressOf());
 		status = winrt_async_await(saveToPackageAsync, throw_on_cancel);
 		if (status != AsyncStatus::Completed)
-			throw std::runtime_error(L("Saving mesh into the 3MF container failed."));
+			throw Slic3r::RuntimeError(L("Saving mesh into the 3MF container failed."));
 		hr = saveToPackageAsync->GetResults();
 
 		Microsoft::WRL::ComPtr<ABI::Windows::Foundation::IAsyncOperation<ABI::Windows::Storage::Streams::IRandomAccessStream*>> generatorStreamAsync;
 		hr = printing3d3mfpackage->SaveAsync(generatorStreamAsync.GetAddressOf());
 		status = winrt_async_await(generatorStreamAsync, throw_on_cancel);
 		if (status != AsyncStatus::Completed)
-			throw std::runtime_error(L("Saving mesh into the 3MF container failed."));
+			throw Slic3r::RuntimeError(L("Saving mesh into the 3MF container failed."));
 		Microsoft::WRL::ComPtr<ABI::Windows::Storage::Streams::IRandomAccessStream> generatorStream;
 		hr = generatorStreamAsync->GetResults(generatorStream.GetAddressOf());
 
@@ -299,7 +299,7 @@ void fix_model_by_win10_sdk(const std::string &path_src, const std::string &path
 			hr = inputStream->ReadAsync(buffer.Get(), 65536 * 2048, ABI::Windows::Storage::Streams::InputStreamOptions_ReadAhead, asyncRead.GetAddressOf());
 			status = winrt_async_await(asyncRead, throw_on_cancel);
 			if (status != AsyncStatus::Completed)
-				throw std::runtime_error(L("Saving mesh into the 3MF container failed."));
+				throw Slic3r::RuntimeError(L("Saving mesh into the 3MF container failed."));
 			hr = buffer->get_Length(&length);
 			if (length == 0)
 				break;
@@ -337,8 +337,8 @@ void fix_model_by_win10_sdk_gui(ModelObject &model_object, int volume_idx)
 
 	// Open a progress dialog.
 	wxProgressDialog progress_dialog(
-		_(L("Model fixing")),
-		_(L("Exporting model...")),
+		_L("Model fixing"),
+		_L("Exporting model") + "...",
 		100, nullptr, wxPD_AUTO_HIDE | wxPD_APP_MODAL | wxPD_CAN_ABORT);
 	// Executing the calculation in a background thread, so that the COM context could be created with its own threading model.
 	// (It seems like wxWidgets initialize the COM contex as single threaded and we need a multi-threaded context).
@@ -365,7 +365,7 @@ void fix_model_by_win10_sdk_gui(ModelObject &model_object, int volume_idx)
 				model_object->add_instance();
 				if (!Slic3r::store_3mf(path_src.string().c_str(), &model, nullptr, false)) {
 					boost::filesystem::remove(path_src);
-					throw std::runtime_error(L("Export of a temporary 3mf file failed"));
+					throw Slic3r::RuntimeError(L("Export of a temporary 3mf file failed"));
 				}
 				model.clear_objects();
 				model.clear_materials();
@@ -380,15 +380,15 @@ void fix_model_by_win10_sdk_gui(ModelObject &model_object, int volume_idx)
 				bool loaded = Slic3r::load_3mf(path_dst.string().c_str(), &config, &model, false);
 			    boost::filesystem::remove(path_dst);
 				if (! loaded)
-	 				throw std::runtime_error(L("Import of the repaired 3mf file failed"));
+	 				throw Slic3r::RuntimeError(L("Import of the repaired 3mf file failed"));
 	 			if (model.objects.size() == 0)
-	 				throw std::runtime_error(L("Repaired 3MF file does not contain any object"));
+	 				throw Slic3r::RuntimeError(L("Repaired 3MF file does not contain any object"));
 	 			if (model.objects.size() > 1)
-	 				throw std::runtime_error(L("Repaired 3MF file contains more than one object"));
+	 				throw Slic3r::RuntimeError(L("Repaired 3MF file contains more than one object"));
 	 			if (model.objects.front()->volumes.size() == 0)
-	 				throw std::runtime_error(L("Repaired 3MF file does not contain any volume"));
+	 				throw Slic3r::RuntimeError(L("Repaired 3MF file does not contain any volume"));
 				if (model.objects.front()->volumes.size() > 1)
-	 				throw std::runtime_error(L("Repaired 3MF file contains more than one volume"));
+	 				throw Slic3r::RuntimeError(L("Repaired 3MF file contains more than one volume"));
 	 			meshes_repaired.emplace_back(std::move(model.objects.front()->volumes.front()->mesh()));
 			}
 			for (size_t i = 0; i < volumes.size(); ++ i) {
