@@ -11,7 +11,6 @@
 
 #include <wx/progdlg.h>
 
-#include "libslic3r/PrintConfig.hpp"
 #include "slic3r/GUI/I18N.hpp"
 #include "slic3r/GUI/GUI.hpp"
 #include "Http.hpp"
@@ -170,6 +169,13 @@ std::string OctoPrint::make_url(const std::string &path) const
     }
 }
 
+SL1Host::SL1Host(DynamicPrintConfig *config) : 
+    OctoPrint(config),
+    authorization_type(dynamic_cast<const ConfigOptionEnum<AuthorizationType>*>(config->option("printhost_authorization_type"))->value),
+    username(config->opt_string("printhost_user")),
+    password(config->opt_string("printhost_password"))
+{
+}
 
 // SL1Host
 const char* SL1Host::get_name() const { return "SL1Host"; }
@@ -191,4 +197,20 @@ bool SL1Host::validate_version_text(const boost::optional<std::string> &version_
     return version_text ? boost::starts_with(*version_text, "Prusa SLA") : false;
 }
 
+void SL1Host::set_auth(Http &http) const
+{
+    switch (authorization_type) {
+    case atKeyPassword:
+        http.header("X-Api-Key", get_apikey());
+        break;
+    case atUserPassword:
+        http.auth_digest(username, password);
+        break;
+    }
+
+    if (! get_cafile().empty()) {
+        http.ca_file(get_cafile());
+    }
+}
+
 }
