@@ -9,6 +9,7 @@
 
 #include <wx/string.h>
 #include <wx/app.h>
+#include <wx/arrstr.h>
 
 #include "libslic3r/PrintConfig.hpp"
 #include "libslic3r/Channel.hpp"
@@ -16,6 +17,7 @@
 #include "Duet.hpp"
 #include "FlashAir.hpp"
 #include "AstroBox.hpp"
+#include "Repetier.hpp"
 #include "../GUI/PrintHostDialogs.hpp"
 
 namespace fs = boost::filesystem;
@@ -47,6 +49,7 @@ PrintHost* PrintHost::get_print_host(DynamicPrintConfig *config)
             case htDuet:      return new Duet(config);
             case htFlashAir:  return new FlashAir(config);
             case htAstroBox:  return new AstroBox(config);
+            case htRepetier:  return new Repetier(config);
             default:          return nullptr;
         }
     } else {
@@ -272,5 +275,4 @@ void PrintHostJobQueue::cancel(size_t id)
     p->channel_cancels.push(id);
 }
 
-
 }
